@@ -24,7 +24,7 @@ import Distribution.Gentoo.Packages
 -- Cabal imports
 import qualified Distribution.Simple.Utils as DSU
 import Distribution.Verbosity(silent)
-import Distribution.Package(PackageIdentifier, packageId)
+import Distribution.Package(packageId)
 import Distribution.InstalledPackageInfo(InstalledPackageInfo)
 import Distribution.Text(display)
 
@@ -39,7 +39,8 @@ import qualified Data.ByteString.Lazy.Char8 as BL8
 import System.FilePath((</>), takeExtension, pathSeparator)
 import System.Directory( canonicalizePath
                        , doesDirectoryExist
-                       , findExecutable)
+                       , findExecutable
+                       , listDirectory )
 import Control.Monad
 
 import Output
@@ -50,7 +51,13 @@ import Output
 
 -- Get only the first line of output
 rawSysStdOutLine     :: FilePath -> [String] -> IO String
-rawSysStdOutLine app = liftM (head . lines) . rawCommand app
+rawSysStdOutLine app args = do
+    out <- rawCommand app args
+    case lines out of
+        [] -> error $ unwords
+            [ "rawSysStdOutLine: Empty output from rawCommand"
+            , show app, show args ]
+        (s:_) -> pure s
 
 rawCommand          :: FilePath -> [String] -> IO String
 rawCommand cmd args = do (out,_,_) <- DSU.rawSystemStdInOut
@@ -84,11 +91,11 @@ ghcRawOut args = ghcLoc >>= flip rawSysStdOutLine args
 -- Cheat with using fromJust since we know that GHC must be in $PATH
 -- somewhere, probably /usr/bin.
 ghcLoc :: IO FilePath
-ghcLoc = liftM fromJust $ findExecutable "ghc"
+ghcLoc = fromJust <$> findExecutable "ghc"
 
 -- The version of GHC installed.
 ghcVersion :: IO String
-ghcVersion = liftM (dropWhile (not . isDigit)) $ ghcRawOut ["--version"]
+ghcVersion = dropWhile (not . isDigit) <$> ghcRawOut ["--version"]
 
 -- The directory where GHC has all its libraries, etc.
 ghcLibDir :: IO FilePath
@@ -100,7 +107,7 @@ ghcPkgRawOut args = ghcPkgLoc >>= flip rawCommand args
 -- Cheat with using fromJust since we know that ghc-pkg must be in $PATH
 -- somewhere, probably /usr/bin.
 ghcPkgLoc :: IO FilePath
-ghcPkgLoc = liftM fromJust $ findExecutable "ghc-pkg"
+ghcPkgLoc = fromJust <$> findExecutable "ghc-pkg"
 
 data ConfSubdir = GHCConfs
                 | GentooConfs
@@ -118,7 +125,7 @@ listConfFiles subdir = do
     let gDir = dir </> subdirToDirname subdir
     exists <- doesDirectoryExist gDir
     if exists
-        then do conts <- getDirectoryContents' gDir
+        then do conts <- listDirectory gDir
                 return $ map (gDir </>)
                     $ filter isConf conts
         else return []
@@ -156,11 +163,9 @@ parse_as_cabal_package cont =
         -- ebuilds that have CABAL_CORE_LIB_GHC_PV set
         -- for this version of GHC will have a .conf
         -- file containing just []
-        [([],_)] -> Nothing
-        rd       -> Just $ CPV $ display $ cfNm rd
-  where
-    cfNm :: [([InstalledPackageInfo], String)] -> PackageIdentifier
-    cfNm = packageId . head . fst . head
+        (([] ,_):_) -> Nothing
+        ((i:_,_):_) -> Just $ CPV $ display
+            $ packageId (i :: InstalledPackageInfo)
 
 -- ghc package text format
 -- "name: zlib-conduit\n
@@ -222,7 +227,7 @@ oldGhcPkgs v =
        -- It would be nice to do this, but we can't assume
        -- some crazy user hasn't deleted one of these dirs
        -- libFronts' <- filterM doesDirectoryExist libFronts
-       liftM notGHC $ checkLibDirs v thisGhc' libFronts
+       notGHC <$> checkLibDirs v thisGhc' libFronts
 
 -- Find packages installed by other versions of GHC in this possible
 -- library directory.
@@ -235,8 +240,17 @@ checkLibDirs v thisGhc libDirs =
 
     isValid dir = any (`isGhcLibDir` dir) libDirs
 
+#if MIN_VERSION_bytestring(0,11,1)
+    -- Correct the path for hadrian-based installations
+    -- See https://github.com/gentoo-haskell/haskell-updater/issues/20
+    thisGhc' = if BS.isSuffixOf (BS.pack "/lib") thisGhc then BS.dropEnd 4 thisGhc else thisGhc
+
     -- Invalid if it's this GHC
+    isInvalid fp = fp == thisGhc' || BS.isPrefixOf (thisGhc' `BS.snoc` pathSeparator) fp
+#else
+    -- Versions of GHC with old 'bytestring' can use the old behavior
     isInvalid fp = fp == thisGhc || BS.isPrefixOf (thisGhc `BS.snoc` pathSeparator) fp
+#endif
 
 -- A valid GHC library directory starting at libdir has a name of
 -- "ghc", then a hyphen and then a version number.
@@ -280,28 +294,28 @@ brokenConfs v =
        vsay v $ unwords ["brokenConfs: resolving Cabal package names to gentoo equivalents."
                         , show (length ghc_pkg_brokens)
                         , "Cabal packages are broken:"
-                        , L.intercalate " " (map unCPV ghc_pkg_brokens)
+                        , unwords $ map unCPV ghc_pkg_brokens
                         ]
 
        (orphan_broken, orphan_confs) <- getOrphanBroken
        vsay v $ unwords [ "brokenConfs: ghc .conf orphans:"
                         , show (length orphan_broken)
                         , "are orphan:"
-                        , L.intercalate " " (map unCPV orphan_broken)
+                        , unwords $ map unCPV orphan_broken
                         ]
 
        installed_but_not_registered <- getNotRegistered v
        vsay v $ unwords [ "brokenConfs: ghc .conf not registered:"
                         , show (length installed_but_not_registered)
                         , "are not registered:"
-                        , L.intercalate " " (map unCPV installed_but_not_registered)
+                        , unwords $ map unCPV installed_but_not_registered
                         ]
 
        registered_twice <- getRegisteredTwice v
        vsay v $ unwords [ "brokenConfs: ghc .conf registered twice:"
                         , show (length registered_twice)
                         , "are registered twice:"
-                        , L.intercalate " " (map unCPV registered_twice)
+                        , unwords $ map unCPV registered_twice
                         ]
 
        let all_broken = concat [ ghc_pkg_brokens
@@ -319,8 +333,8 @@ brokenConfs v =
 -- Return the closure of all packages affected by breakage
 -- in format of ["name-version", ... ]
 getBrokenGhcPkg :: IO [CabalPV]
-getBrokenGhcPkg = liftM (map CPV . words)
-                  $ ghcPkgRawOut ["check", "--simple-output"]
+getBrokenGhcPkg = map CPV . words
+                  <$> ghcPkgRawOut ["check", "--simple-output"]
 
 getOrphanBroken :: IO ([CabalPV], [FilePath])
 getOrphanBroken = do
@@ -331,9 +345,9 @@ getOrphanBroken = do
        confs_to_pkgs <- resolveFiles registered_confs
        let (conf_files, _conf_pkgs) = unzip confs_to_pkgs
            orphan_conf_files = registered_confs L.\\ conf_files
-       orphan_packages <- liftM catMaybes $
+       orphan_packages <- fmap catMaybes $
                               forM orphan_conf_files $
-                                  liftM parse_as_ghc_package . BS.readFile
+                                  fmap parse_as_ghc_package . BS.readFile
        return (orphan_packages, orphan_conf_files)
 
 -- Return packages, that seem to have
@@ -368,5 +382,5 @@ getRegisteredTwice v = do
 allInstalledPackages :: IO [Package]
 allInstalledPackages = do libDir <- ghcLibDir
                           let libDir' = BS.pack libDir
-                          liftM notGHC $ pkgsHaveContent
+                          fmap notGHC $ pkgsHaveContent
                                        $ hasDirMatching (==libDir')
